package cz.muni.fi.pa165.authorization.server.rest;

import cz.muni.fi.pa165.authorization.server.api.UserApiDelegate;
import cz.muni.fi.pa165.authorization.server.model.CreationUserDto;
import cz.muni.fi.pa165.authorization.server.model.LoginDto;
import cz.muni.fi.pa165.authorization.server.model.LoginResponse;
import cz.muni.fi.pa165.authorization.server.model.UserDto;
import cz.muni.fi.pa165.authorization.server.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
public class UserController implements UserApiDelegate {

    private final UserService userService;

    @Autowired
    public UserController(UserService userService) {
        this.userService = userService;
    }

    @Override
    public ResponseEntity<UserDto> getUserById(Long id) {
        return ResponseEntity.ok(userService.getUserById(id));
    }

    @Override
    public ResponseEntity<UserDto> createUser(CreationUserDto creationUserDto) {
        return ResponseEntity.ok(userService.createUser(creationUserDto));
    }

    @Override
    public ResponseEntity<UserDto> updateUserById(Long id, CreationUserDto creationUserDto) {
        return ResponseEntity.ok(userService.updateUserById(id, creationUserDto));
    }

    @Override
    public ResponseEntity<Void> deleteUserById(Long id) {
        userService.deleteUserById(id);
        return ResponseEntity.ok(null);
    }

    @Override
    public ResponseEntity<LoginResponse> login(LoginDto loginDto) {
        return ResponseEntity.ok(userService.login(loginDto));
    }

    @Override
    public ResponseEntity<List<UserDto>> getAllUsers() {
        return ResponseEntity.ok(userService.getAllUsers());
    }
}
