package cz.muni.fi.pa165.core.facade.flight;

import cz.muni.fi.pa165.core.exceptions.ResourceNotFoundException;
import cz.muni.fi.pa165.core.mapper.FlightMapper;
import cz.muni.fi.pa165.core.model.FlightDto;
import cz.muni.fi.pa165.core.model.NewFlightDtoRequest;
import cz.muni.fi.pa165.core.service.flight.FlightService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class FlightFacadeImpl implements FlightFacade<Long> {

    private final FlightService flightService;
    private final FlightMapper flightMapper;

    @Autowired
    public FlightFacadeImpl(FlightService flightService, FlightMapper flightMapper) {
        this.flightService = flightService;
        this.flightMapper = flightMapper;
    }

    @Override
    public FlightDto save(NewFlightDtoRequest newFlightDtoRequest) {
        var entityToSave = flightMapper.toEntityFromNewRequest(newFlightDtoRequest);
        var savedEntity = flightService.save(entityToSave);
        return flightMapper.toDto(savedEntity);
    }

    @Override
    public Optional<FlightDto> findById(Long id) {
        var foundFlightEntity = flightService.findById(id)
                .orElseThrow(() -> new ResourceNotFoundException("Not Found."));

        return Optional.ofNullable(flightMapper.toDto(foundFlightEntity));
    }

    @Override
    public List<FlightDto> findAll() {
        return null;
    }

    @Override
    public void deleteById(Long id) {

    }

    @Override
    public void deleteAll() {

    }
}
