package cz.muni.fi.pa165.core.facade.steward;

import cz.muni.fi.pa165.core.data.domain.FlightSteward;
import cz.muni.fi.pa165.core.data.domain.Steward;
import cz.muni.fi.pa165.core.exceptions.ResourceNotFoundException;
import cz.muni.fi.pa165.core.mapper.StewardMapper;
import cz.muni.fi.pa165.core.model.FlightDto;
import cz.muni.fi.pa165.core.model.NewStewardDtoRequest;
import cz.muni.fi.pa165.core.model.StewardDto;
import cz.muni.fi.pa165.core.service.flight.FlightService;
import cz.muni.fi.pa165.core.service.steward.StewardService;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class StewardFacadeImpl implements StewardFacade<Long> {

    private final StewardService stewardService;
    private final FlightService flightService;
    private final StewardMapper stewardMapper;

    @Autowired
    public StewardFacadeImpl(StewardService stewardService, FlightService flightService, StewardMapper stewardMapper) {
        this.stewardService = stewardService;
        this.flightService = flightService;
        this.stewardMapper = stewardMapper;
    }

    @Override
    public StewardDto save(NewStewardDtoRequest newStewardDtoRequest) {
        var entityToSave = stewardMapper.toEntityFromNewRequest(newStewardDtoRequest);
        var savedEntity = stewardService.save(entityToSave);
        return stewardMapper.toDto(savedEntity);
    }

    @Override
    public Optional<StewardDto> findById(Long id) {
        return Optional.empty();
    }

    @Override
    public List<StewardDto> findAll() {
        return null;
    }

    @Override
    public void deleteById(Long id) {

    }

    @Override
    public void deleteAll() {

    }

    @Override
    public StewardDto assignStewardFlight(Long stewardId, Long flightId) {
        var stewardEntity = stewardService.findByIdWithFlights(stewardId)
                .orElse(stewardService.findById(stewardId)
                        .orElseThrow(() -> new ResourceNotFoundException("Not Found.")));

        var flightEntity = flightService.findById(flightId)
                .orElseThrow(() -> new ResourceNotFoundException("Not Found."));

        var flightSteward = new FlightSteward();
        flightSteward.setSteward(stewardEntity);
        flightSteward.setFlight(flightEntity);
        stewardService.saveFlightStewards(flightSteward);

        var stewardDto = stewardMapper.toDto(stewardEntity);

        return stewardDto;
    }
}
