package cz.muni.fi.pa165.report.server;

import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.test.web.servlet.MockMvc;

import java.io.InputStream;

import static org.assertj.core.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 * Integration tests. Run by "maven verify".
 */
@SpringBootTest
@AutoConfigureMockMvc
class ReportIT {

    private static final Logger log = LoggerFactory.getLogger(ReportIT.class);

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    ObjectMapper objectMapper;

    @Test
    void getReportFlightByIdTest() throws Exception {
        log.debug("getReportFlightByIdTest() running");

        ClassLoader classLoader = getClass().getClassLoader();

        try(InputStream inputStream = classLoader.getResourceAsStream("sample.pdf")){
            assert inputStream != null;
            byte[] bytes = inputStream.readAllBytes();
            ByteArrayResource resource = new ByteArrayResource(bytes);
            var response = mockMvc.perform(get("/api/reports/flight/1"))
                    .andExpect(status().isOk())
                    .andReturn().getResponse().getContentAsByteArray();
            assertThat(response).isEqualTo(resource.getByteArray());
            log.debug("response: {}", response);
        }

    }

    @Test
    void getReportAirportByIdTest() throws Exception {
        log.debug("getReportFlightByIdTest() running");

        ClassLoader classLoader = getClass().getClassLoader();

        try(InputStream inputStream = classLoader.getResourceAsStream("sample.pdf")){
            assert inputStream != null;
            byte[] bytes = inputStream.readAllBytes();
            ByteArrayResource resource = new ByteArrayResource(bytes);
            var response = mockMvc.perform(get("/api/reports/airport/1"))
                    .andExpect(status().isOk())
                    .andReturn().getResponse().getContentAsByteArray();
            assertThat(response).isEqualTo(resource.getByteArray());
            log.debug("response: {}", response);
        }

    }

    @Test
    void getReportAirplaneByIdTest() throws Exception {
        log.debug("getReportFlightByIdTest() running");

        ClassLoader classLoader = getClass().getClassLoader();

        try(InputStream inputStream = classLoader.getResourceAsStream("sample.pdf")){
            assert inputStream != null;
            byte[] bytes = inputStream.readAllBytes();
            ByteArrayResource resource = new ByteArrayResource(bytes);
            var response = mockMvc.perform(get("/api/reports/airplane/1"))
                    .andExpect(status().isOk())
                    .andReturn().getResponse().getContentAsByteArray();
            assertThat(response).isEqualTo(resource.getByteArray());
            log.debug("response: {}", response);
        }

    }
}
